{{

}}

CON

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq
  MICROSECOND = CLK_FREQ / 1_000_000
  MILLISECOND = CLK_FREQ / 1_000

  SCALED_MULTIPLIER = 1000
  F_SCALED_MULTIPLIER = float(SCALED_MULTIPLIER)
  SCALED_POSITION = 3
  OUTPUT_SIZE = 8 'three digits each side of decimal point, dec point and negative sign
  
  START_GRID = -10
  END_GRID = 10
  GRID_ROWS = END_GRID - START_GRID
  MAX_ROW_INDEX = GRID_ROWS - 1
  TEST_POINTS = GRID_ROWS * 4
  MAX_POINT_INDEX = TEST_POINTS - 1

  'dataSet enumeration
  #0, ATAN2_INT, DISTANCE_INT, FIND_X_INT, FIND_Y_INT
      ATAN2_FLOAT, DISTANCE_FLOAT, FIND_X_FLOAT, FIND_Y_FLOAT

  MAX_DATA_SET_INDEX = DISTANCE_FLOAT
  DATA_SETS = MAX_DATA_SET_INDEX + 1
  
VAR

  long fX[TEST_POINTS], fY[TEST_POINTS]
  long fXNew[TEST_POINTS], fYNew[TEST_POINTS]
  long floatAngle[TEST_POINTS]
  long floatDistance[TEST_POINTS]
  long x[TEST_POINTS], y[TEST_POINTS]
  long xNew[TEST_POINTS], yNew[TEST_POINTS]
  long intAngle[TEST_POINTS]
  long intDistance[TEST_POINTS]
  long timer[DATA_SETS]
  
OBJ

  Pst : "Parallax Serial Terminal"
  Math : "SL32_INTEngine_2"
  F32 : "F32"
  Format : "StrFmt"
         
PUB Main : localIndex | xIndex, yIndex, xPoint, yPoint

  Pst.Start(115200)         'start a terminal Object (rxpin, txpin, mode, baud rate)

  repeat
    Pst.str(string(11, 13, "Press any key to start."))
    localIndex := Pst.RxCount
    waitcnt(clkfreq / 2 + cnt)
  until localIndex

  Pst.RxFlush
  Pst.Clear
  Pst.Home
  F32.Start

  xIndex := 0
  yIndex := 0
  
  Pst.str(string(11, 13, "           ****** Scroll down to see times. ******", 11, 13))
  localIndex := 0
  yPoint := END_GRID * SCALED_MULTIPLIER
  xPoint := START_GRID * SCALED_MULTIPLIER
  longfill(@fY, F32.FFloat(yPoint), GRID_ROWS)
  longfill(@y, yPoint, GRID_ROWS)
  repeat GRID_ROWS
    x[localIndex] := xPoint
    fX[localIndex++] := F32.FFloat(xPoint)
    xPoint += SCALED_MULTIPLIER
  longfill(@fX[localIndex], F32.FFloat(xPoint), GRID_ROWS)
  longfill(@x[localIndex], xPoint, GRID_ROWS)
  
  repeat GRID_ROWS
    y[localIndex] := yPoint
    fY[localIndex++] := F32.FFloat(yPoint)
    yPoint -= SCALED_MULTIPLIER
  longfill(@fY[localIndex], F32.FFloat(yPoint), GRID_ROWS)
  longfill(@y[localIndex], yPoint, GRID_ROWS)
  
  repeat GRID_ROWS
    x[localIndex] := xPoint
    fX[localIndex++] := F32.FFloat(xPoint)
    xPoint -= SCALED_MULTIPLIER
  longfill(@fX[localIndex], F32.FFloat(xPoint), GRID_ROWS)
  longfill(@x[localIndex], xPoint, GRID_ROWS)
  
  repeat GRID_ROWS
    y[localIndex] := yPoint
    fY[localIndex++] := F32.FFloat(yPoint)
    yPoint += SCALED_MULTIPLIER
  {Pst.str(string(11, 13, "test coordinates = ", 11, 13))
  repeat localIndex from 0 to MAX_POINT_INDEX
    Pst.ClearEnd
    Pst.Newline
    DecPoint(x[localIndex], OUTPUT_SIZE, SCALED_POSITION)
    Pst.Char(",")
    Pst.Char(" ")
    DecPoint(y[localIndex], OUTPUT_SIZE, SCALED_POSITION)
  }
  timer[ATAN2_INT] := -cnt

  repeat localIndex from 0 to MAX_POINT_INDEX
    intAngle[localIndex] := Math.polarAngle(y[localIndex], x[localIndex])

  timer[ATAN2_INT] += cnt

  timer[DISTANCE_INT] := -cnt

  repeat localIndex from 0 to MAX_POINT_INDEX
    intDistance[localIndex] := Math.polarRadius(y[localIndex], x[localIndex])

  timer[DISTANCE_INT] += cnt

  timer[FIND_X_INT] := -cnt

  repeat localIndex from 0 to MAX_POINT_INDEX
    xNew[localIndex] := Math.cos(intAngle[localIndex], intDistance[localIndex])

  timer[FIND_X_INT] += cnt

  timer[FIND_Y_INT] := -cnt

  repeat localIndex from 0 to MAX_POINT_INDEX
    yNew[localIndex] := Math.sin(intAngle[localIndex], intDistance[localIndex])

  timer[FIND_Y_INT] += cnt

  timer[ATAN2_FLOAT] := -cnt

  repeat localIndex from 0 to MAX_POINT_INDEX
    floatAngle[localIndex] := F32.ATan2(fY[localIndex], fX[localIndex])

  timer[ATAN2_FLOAT] += cnt

  timer[DISTANCE_FLOAT] := -cnt

  repeat localIndex from 0 to MAX_POINT_INDEX
    floatDistance[localIndex] := ComputeDistance2D(fY[localIndex], fX[localIndex])

  timer[DISTANCE_FLOAT] += cnt
 
  timer[FIND_X_FLOAT] := -cnt

  repeat localIndex from 0 to MAX_POINT_INDEX
    fXNew[localIndex] := F32.FMul(floatDistance[localIndex], F32.cos(floatAngle[localIndex]))

  timer[FIND_X_FLOAT] += cnt

  timer[FIND_Y_FLOAT] := -cnt

  repeat localIndex from 0 to MAX_POINT_INDEX
    fYNew[localIndex] := F32.FMul(floatDistance[localIndex], F32.sin(floatAngle[localIndex]))

  timer[FIND_Y_FLOAT] += cnt

  DisplayResults

  Pst.str(string(11, 13, "Integer Math: atan2 time = "))
  DecPoint(timer[ATAN2_INT] / MICROSECOND, 7, 3) 
  Pst.str(string(" ms, distance time = "))
  DecPoint(timer[DISTANCE_INT] / MICROSECOND, 7, 3)
  Pst.str(string(" ms, find x time = "))
  DecPoint(timer[FIND_X_INT] / MICROSECOND, 7, 3) 
  Pst.str(string(" ms, find y time = "))
  DecPoint(timer[FIND_Y_INT] / MICROSECOND, 7, 3) 
  Pst.str(string(" ms"))
  Pst.str(string(11, 13, "  Float Math: atan2 time = "))
  DecPoint(timer[ATAN2_FLOAT] / MICROSECOND, 7, 3) 
  Pst.str(string(" ms, distance time = "))
  DecPoint(timer[DISTANCE_FLOAT] / MICROSECOND, 7, 3) 
  Pst.str(string(" ms, find x time = "))
  DecPoint(timer[FIND_X_FLOAT] / MICROSECOND, 7, 3) 
  Pst.str(string(" ms, find y time = "))
  DecPoint(timer[FIND_Y_FLOAT] / MICROSECOND, 7, 3) 
  Pst.str(string(" ms"))
  repeat

PUB DisplayResults : localIndex
                         '          1         2         3         4         5         6         7         8         9
                         '0123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456
  Pst.str(string(11, 13, " exact | int  | int | recalculated  |float | float  | recalculated"))
  Pst.str(string(11, 13, "(x, y) | dist |angle| integer x, y  | dist | angle  | float x, y", 11, 13))
                         '-10,-10|12.345| -123|-12.345,-12.345|12.345|-123.456|
  repeat localIndex from 0 to MAX_POINT_INDEX
    Pst.ClearEnd
    Pst.Newline 
    DecPoint(x[localIndex] / SCALED_MULTIPLIER, 3, 0) 
    Pst.Char(",")
    DecPoint(y[localIndex] / SCALED_MULTIPLIER, 3, 0) 
    Pst.PositionX(7)
    Pst.Char("|")
    DecPoint(intDistance[localIndex], 6, SCALED_POSITION) 
    Pst.PositionX(14)
    Pst.Char("|")
    Pst.Char(" ")
    DecPoint(intAngle[localIndex], 4, 0) 
    Pst.PositionX(20)
    Pst.Char("|")
    DecPoint(xNew[localIndex], 7, SCALED_POSITION) 
    Pst.Char(",")
    DecPoint(yNew[localIndex], 7, SCALED_POSITION) 
    Pst.PositionX(36)
    Pst.Char("|")
    DecPoint(F32.FRound(floatDistance[localIndex]), 6, SCALED_POSITION)
    Pst.PositionX(43)
    Pst.Char("|")
    DisplayFloatAngle(floatAngle[localIndex])
    Pst.PositionX(52)
    Pst.Char("|")
    DecPoint(F32.FRound(fXNew[localIndex]), 7, SCALED_POSITION)
    Pst.Char(",")
    DecPoint(F32.FRound(fYNew[localIndex]), 7, SCALED_POSITION)
    
PUB DisplayFloatDistance(fValue)

  DecPoint(F32.FRound(fValue), OUTPUT_SIZE, SCALED_POSITION)
  
PUB DisplayFloatAngle(fValue)

  DecPoint(F32.FRound(F32.FMul(F32.Degrees(fValue), F_SCALED_MULTIPLIER)), OUTPUT_SIZE, SCALED_POSITION)
  
PUB DecPoint(value, length, places) | localBuffer[3]
'' Lock should be set prior to calling this method.

  result := Format.Fdec(@localBuffer, value, length, places)
  byte[result] := 0 ' probably not needed
  Pst.Str(@localBuffer)
      
PUB ComputeDistance2D(fLocalX, fLocalY) 

  result := F32.FSqr(F32.FAdd(F32.FMul(fLocalX, fLocalX), F32.FMul(fLocalY, fLocalY)))
